package ucl.context;

import java.io.File;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;

import com.vividsolutions.jts.geom.Point;

import repast.simphony.context.Context;
import repast.simphony.context.space.graph.NetworkFactoryFinder;
import repast.simphony.context.space.grid.GridFactoryFinder;
import repast.simphony.dataLoader.ContextBuilder;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.environment.RunState;
import repast.simphony.parameter.Parameters;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.graph.Network;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridBuilderParameters;
import repast.simphony.space.grid.RandomGridAdder;
import repast.simphony.space.grid.StickyBorders;
import ucl.entities.Settlement;
import ucl.entities.SettlementEdge;
import ucl.entities.Agent;
import ucl.model.AgentSettlementModel;
import ucl.model.SettlementModel;
import ucl.output.SettlementOutputter;
import ucl.output.Outputter;
import ucl.output.ShapeFileLoader;

/**
 * Class to instantiate the simulation.
 * 
 * @author 
 *
 * @param <T>
 */
public class ContextCreator<T> implements ContextBuilder<T>{
	/**List containing simulation settlements **/
	public static List<Settlement> cities=new ArrayList<Settlement>();
	
	/**Network used for settlement connections**/
	private Network<Settlement> cNetwork;
	
	/**
	 * Context for building and instantiating the simulation
	 */
	public Context build(Context<T> context) {
		ContextCreator.cities.clear();
		cNetwork = 
				(Network<Settlement>) NetworkFactoryFinder.createNetworkFactory(null).
				createNetwork("Flow",context,true);
		
		
		Outputter.runNumber=RunState.getInstance().getRunInfo().getRunNumber();
		Outputter.started=false;
		Parameters p = RunEnvironment.getInstance().getParameters();
		
		String loc = p.getValueAsString("scenarioData");
		double endTick=((Double)p.getValue("endTick")).intValue();
		int people=p.getInteger("peoplePerSite");
		double operationCost=p.getDouble("operationCost");
		double moveProbability=p.getDouble("moveProbability");
		
		createSettlement(loc,context,p);
		createPeople(context,people,p);
		createNetworkCities();
			
		SettlementModel.operationCost=operationCost;
		AgentSettlementModel.moveProbability=moveProbability;
		
		RunEnvironment.getInstance().endAt(endTick);
		
		return context;
	}
	
	/**
	 * Method for city creation.
	 * @param location the location of the data file
	 * @param context the simulation context that contains simulation data
	 */
	public void createSettlement(String location,Context context,Parameters par){
		 cities.clear();
		 
		 List<Map<String,Object>> data=ShapeFileLoader.shpLoader(new File(location));
		 double bMultiplier=par.getDouble("benefitMultiplier");
		 for(Map m : data){
			 Settlement c = new Settlement();
			 
			 Point p=(Point) m.get("the_geom");
			 int id=-1;
			 
			 try{
				 id=(Integer)m.get("CAT");
			 }
			 catch(java.lang.ClassCastException e){
				 id=((Long)m.get("CAT")).intValue();
			 }
			
			 String name=(String)m.get("SITE_NAME");
			 double cBenefit=(Double)m.get("CBENEFIT");
			 double eS=(Double)m.get("SIZE");
			 double eX=(Double)m.get("EXOGENOUS")+bMultiplier;
			 double eX_sd=(Double)m.get("EXOG_SD");
			 double x=p.getX();
			 double y=p.getY();
			 
	//		 double bX=(Double)m.get("X");
	//		 double bY=(Double)m.get("Y");
			
			 for(int i=1; i <= data.size(); i++){
				 double v=(Double)m.get("POINT"+Integer.toString(i));
				 c.getTravelCosts().put(i, v);
			 }
			 c.setPoint(p);
			 c.setX(x);
			 c.setY(y);
			 c.setName(name);
			 c.setIdN(id);
			 c.setSize(eS);
			 c.setBenefitMultiplier(eX);
			 c.setBenefitMultiplier_sd(eX_sd);
			 c.setSettlementBenefit(cBenefit);
			 
			 double alpha=((Double)par.getValue("alpha")).doubleValue();
			 double beta=((Double)par.getValue("beta")).doubleValue();
			 
			 c.setAlpha(alpha);
			 c.setBeta(beta);
			 
			 context.add(c);
	//		 grid.moveTo(c, (int)Math.rint(bX*100),(int)Math.rint(bY*100));
			 cities.add(c);
		 }
		 SettlementOutputter co = new SettlementOutputter();
		 context.add(co);	
	}
	
	/**
	 * Method to create people in the simulation.
	 * @param context the simulation context that contains simulation data
	 * @param people the number of people to create
	 */
	public void createPeople(Context context, int people, Parameters pp){
		Iterator<Settlement> ic = cities.iterator();
		
		int ii=0;
		while(ic.hasNext()){
			Settlement c = ic.next();
			for(int i=0; i < people; i++){
				double bm=c.benefitMultiplier;
				double sdbm=c.benefitMultiplier_sd;
				double nbm=RandomHelper.createNormal(bm, sdbm).nextDouble();
				Agent p = new Agent(ii,c.getIdN());
				p.setBenefitMultiplier(Math.max(nbm,0.0));
				c.getPeople().add(p);
				p.setSettlement(c);
				p.setOriginalSettlement(c.idN);
				context.add(p);
				ii++;
			}
		}
	}
	
	/**
	 * The network creator for sites in study area. Each settlement will be connected to all other settlements.
	 */
	public void createNetworkCities(){
		
		for(int ii=0; ii < cities.size();ii++){
			Settlement c = cities.get(ii);
			cities.remove(c);
			for(int i=0; i < cities.size(); i++){
				Settlement cc = cities.get(i);
				SettlementEdge cef = new SettlementEdge(c,cc);
				SettlementEdge cef2 = new SettlementEdge(c,cc);
				cNetwork.addEdge(cef);
	//			dNetwork.addEdge(cef2);
			}
			cities.add(ii,c);
		}
	}

}
