package ucl.model;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.environment.RunState;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.graph.Network;
import ucl.context.ContextCreator;
import ucl.entities.Settlement;
import ucl.entities.SettlementEdge;
import ucl.entities.Agent;

/**
 * Model used to calculate person-level values and determining if a city is of benefit
 * or not to an agent based on social and environmental inputs
 * 
 * @author 
 *
 */
public class AgentSettlementModel implements Model{
	
	/**The probability of moving to a given settlement*/
	public static double moveProbability=1.0d;
	
	
	/**
	 * Step #2 of the overall model.
	 */
	public void run(Object p){
//		this.totalCities.clear();
		if(RunEnvironment.getInstance().getCurrentSchedule().getTickCount()==1.0){
			Iterator<Settlement>ic = ContextCreator.cities.iterator();
			while(ic.hasNext()){
				Settlement c = ic.next();
				c.getModel().calculateSocioEnvironment(c, (Agent)p);
			}
			return;
		}
			
		flowFromTowns((Agent)p);
	}
	
	
	/**
	 * Step #3&4 of the overall model.
	 */
	public void runSecond(Object p){
		if(RunEnvironment.getInstance().getCurrentSchedule().getTickCount()==1.0)
			return;
		determineRelocate((Agent)p);
	}
	
	/**
	 * This determines probability a person will visit 
	 * and thus benefit a site via interaction. This calculates the person's flow
	 * to a given place (i.e., going to a given place over a time span) based on total
	 * overall flow to all possible settlements.
	 * 
	 * @param p a person
	 */
	public void flowFromTowns(Agent p){
		p.setFlow(0.0);
		Iterator<Settlement> ic=ContextCreator.cities.iterator();
		while(ic.hasNext()){
			Settlement c = ic.next();
//			if(c==p.settlement)
//				continue;
			c.getModel().calculateSocioEnvironment(c, p);
		
			double rC=c.rangeOfChoice.get(p);
			double tC=0;
			if(p.settlement.getTravelCosts().get(c.idN)>0)
				tC=Math.log(p.settlement.getTravelCosts().get(c.idN));
			
			double flow=Math.pow(rC,c.alpha)*Math.exp(-c.beta*
					tC);
			
			double cost=SettlementModel.operationCost*tC;
			flow=flow-cost;
			p.setFlow(p.getFlow()+flow);
			p.flowToSettlementProbability.put(c, flow);
		}
		p.settlement.flow+=p.flow;
		SettlementModel.totalFlow+=p.flow;
	}
	
	/**
	 * This is used in the probabilityOfVisit(Person p) method as the denominator
	 * to give the total person flow value for given settlements. This allows
	 * to determine the proportion of flow and probability of a person to a given settlement.
	 * 
	 * @param p a person
	 * @return the total flow to settlements for person p
	 */
//	public double totalCities(Person p){
//		double result=0;
		
//		Iterator<Settlement> ic = ContextCreator.cities.iterator();
//		while(ic.hasNext()){
//			Settlement c = ic.next();
//			if(c==p.city)
//				continue;
//			double bv=c.rangeOfChoice.get(p);
//			double v=Math.pow(bv,c.alpha)*Math.exp(-c.beta*
//					Math.log(p.city.getTravelCosts().get(c.idN)));
//			result+=v;
//		}
		
//		return result;
//	}
		
	/**
	 * Based on overall net flow and probability of relocation, a person determines
	 * if he/she will relocate to a new settlement and which settlement that will be.
	 * 
	 * @param p a person p
	 */
	public void determineRelocate(Agent p){
		
		Network flowN = (Network) RunState.getInstance().getMasterContext().getProjection("Flow");
		int peoplePerSite=RunEnvironment.getInstance().getParameters().getInteger("peoplePerSite");
		double eFlow=SettlementModel.totalFlow/(double)(peoplePerSite*ContextCreator.cities.size());
		
		double r=RandomHelper.nextDouble();
		
		if((p.flow<0 || p.flow<eFlow) &&( r < moveProbability)){
			Settlement cc=relocate(p);
			
			if(p.settlement.peopleCities.containsKey(p.originalSettlement)){
				p.settlement.peopleCities.put(p.originalSettlement, 
						p.settlement.peopleCities.get(p.originalSettlement)-1);
				
				if(p.settlement.peopleCities.get(p.originalSettlement)<=0)
					p.settlement.peopleCities.remove(p.originalSettlement);
			}
			SettlementEdge e=(SettlementEdge) flowN.getEdge(p.settlement,cc);
			e.setUsed(e.getUsed()+1);
			p.settlement.getPeople().remove(p);
			p.settlement=cc;
			cc.getPeople().add(p);
			if(cc.getPeopleCities().containsKey(p.originalSettlement))
				cc.getPeopleCities().put(p.originalSettlement, cc.getPeopleCities().get(p.originalSettlement)+1);
			else
				cc.getPeopleCities().put(p.originalSettlement,1);
			
		}
		else{
			return;	
		}
	}
	
	/**
	 * This method selects the settlement of relocation based on distance and its 
	 * flow (e.g., economic) and social benefits (i.e., other people from the same home or initial settlement)
	 * 
	 * @param p a person
	 * @return a settlement to relocate to.
	 */
	public Settlement relocate(Agent p){
		Network flowN = (Network) RunState.getInstance().getMasterContext().getProjection("Flow");
		Settlement c = p.settlement;
		Iterator ei=flowN.getOutEdges(c).iterator();
		List<Double> lengthPositive=new ArrayList<Double>();
		List<Double> lengthNegative=new ArrayList<Double>();
	
		Map<Double,List<Settlement>>distances=new HashMap<Double,List<Settlement>>();
	
		while(ei.hasNext()){
			SettlementEdge ce = (SettlementEdge) ei.next();
			Settlement s=(Settlement) ce.getSource();
			Settlement t=(Settlement) ce.getTarget();
			if(t==p.settlement)
				continue;
			double pLength=s.getTravelCosts().get(t.idN);
			double wL=0.0;
			double nP=1.0;
			if(t.peopleCities.containsKey(p.originalSettlement))
				nP=1+(double)(t.peopleCities.get(p.originalSettlement)/(double)t.people.size());
			if(t.flow>0)
				wL=pLength/(t.flow*p.benefitMultiplier*nP);
			else 
				wL=(Math.abs(t.flow)*pLength)/(p.benefitMultiplier*nP);

			if(t.flow>0)
				lengthPositive.add(wL);
			else
				lengthNegative.add(wL);
			
			if(distances.containsKey(wL)){
				List<Settlement> sets = distances.get(wL);
				sets.add(t);
				distances.put(wL, sets);
			}
			else{
				List<Settlement> sets = new ArrayList<Settlement>();
				sets.add(t);
				distances.put(wL,sets);
			}
		}
		Collections.sort(lengthPositive);
		Collections.sort(lengthNegative);
		
		Settlement citi=null;
		if(!lengthPositive.isEmpty()){
			List<Settlement> sets=distances.get(lengthPositive.get(0));
			int n=RandomHelper.nextIntFromTo(0, sets.size()-1);
			citi=sets.get(n);
			
		}
		else{
			List<Settlement> sets=distances.get(lengthNegative.get(0));
			int n=RandomHelper.nextIntFromTo(0, sets.size()-1);
			citi=sets.get(n);
		}
		
		return citi;
	}
}
