package ucl.model;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.HashMap;

import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.parameter.ParametersCreator;
import ucl.context.ContextCreator;
import ucl.entities.Agent;
import ucl.entities.Settlement;

import org.junit.Test;

/**
 * Test class for agent city model and agent behaviour.
 * 
 * @author 
 *
 */
public class TestAgentSettlementModel {

	// necessary to allow agents to be created and tested in the model
		static {
			ParametersCreator creator = new ParametersCreator();
			creator.addParameter("epsilon", double.class, 1.0, false);
			RunEnvironment.init(null, null, creator.createParameters(), false);
		}
		
		/**
		 * Create people
		 * @param ps people list
		 * @param n number of people
		 * @param cn id number for site
		 * @param s
		 */
		public void createPeople(List<Agent>ps,int n, int cn,Settlement s){
			for(int i=0; i < n; i++){
				Agent p = new Agent(i,cn);
				p.benefitMultiplier=1.0;
				p.flow=1.0;
				p.settlement=s;
				ps.add(p);
			}
		}
		
		/**
		 * Create a settlement
		 */
		public void createSettlement(){
			
			Map<Integer,Double> costs1=new HashMap<Integer,Double>();
			Map<Integer,Double> costs2=new HashMap<Integer,Double>();
			costs1.put(0, 0.0);
			costs1.put(1, 5.0);
			costs2.put(0, 5.0);
			costs2.put(1, 0.0);
			
			List<Agent>totalP=new ArrayList<Agent>();
			
			for(int i=0; i < 2; i++) {
				Settlement s = new Settlement();
				s.setAlpha(1.0);
				s.setBeta(1.0);
				s.setBenefitMultiplier(1.0);
				if(i==1)
					s.flow=-10.0;
				else
					s.flow=10.0;
				s.setIdN(1);
				s.setModel(new SettlementModel());
				s.setName("name");
				List<Agent> ps = new ArrayList<Agent>();
				createPeople(ps,10,i,s);
				totalP.addAll(ps);
				s.setPeople(ps);
				if(i==0)
					s.setTravelCosts(costs1);
				else
					s.setTravelCosts(costs2);
				ContextCreator.cities.add(s);
			}
			
			for(int i=0; i < totalP.size(); i++){
				Agent p = totalP.get(i);
				for(int t=0; t < ContextCreator.cities.size(); t++){
					Settlement s = ContextCreator.cities.get(t);
					s.rangeOfChoice.put(p, 1.0);
				}
			}
		}
		
		/**
		 * Test probabilityOfVisit
		 */
		@Test
		public void testProbabilityOfVisit(){
			ContextCreator.cities.clear();
			AgentSettlementModel acm = new AgentSettlementModel();
			SettlementModel.operationCost=0.0;
			createSettlement();
			Iterator<Settlement>si=ContextCreator.cities.iterator();
			SettlementModel cm = new SettlementModel();
			
			Settlement s = si.next();
			cm.clearFlow(s);
			List<Agent>ps = s.getPeople();
			for(Agent p:ps){
				acm.flowFromTowns(p);
				assertTrue(p.flow==2.0);
			}
			assertTrue(s.flow==20.0);
			
			
		}
		
		/**
		 * Test determineRelocate().
		 */
		@Test
		public void testDetermineRelocate(){
			AgentSettlementModel acm = new AgentSettlementModel();
			createSettlement();
			Iterator<Settlement>si=ContextCreator.cities.iterator();
			while(si.hasNext()){
				Settlement s = si.next();
				List<Agent>ps = s.getPeople();
				for(Agent p:ps){
		//			acm.determineRelocate(p);
		//			assertTrue(p.city==s);
				}
			}
			
		}
}
